@@ -2,6 +2,8 @@ from time import time, sleep
 import logging
 from queue import Queue
 from pymodbus.client.sync import ModbusTcpClient, ModbusSocketFramer
+from pymodbus import exceptions
+from SungrowModbusTcpClient import SungrowModbusTcpClient
 
 DEFAULT_SCAN_RATE_S = 5
 DEFAULT_SCAN_BATCHING = 100
@@ -10,7 +12,7 @@ DEFAULT_WRITE_SLEEP_S = 0.05
 
 class modbus_interface():
 
-    def __init__(self, ip, port=502, update_rate_s=DEFAULT_SCAN_RATE_S):
+    def __init__(self, ip, port=502, update_rate_s=DEFAULT_SCAN_RATE_S, variant=None):
         self._ip = ip
         self._port = port
         # This is a dict of sets. Each key represents one table of modbus registers.
@@ -22,12 +24,21 @@ class modbus_interface():
 
         self._planned_writes = Queue()
         self._writing = False
+        self._variant = variant
 
     def connect(self):
         # Connects to the modbus device
-        self._mb = ModbusTcpClient(self._ip, self._port,
-                                  framer=ModbusSocketFramer, timeout=1,
-                                  RetryOnEmpty=True, retries=1)
+        if self._variant == 'sungrow':
+            # Some later versions of the sungrow inverter firmware encrypts the payloads of
+            # the modbus traffic. https://github.com/rpvelloso/Sungrow-Modbus is a drop-in
+            # replacement for ModbusTcpClient that manages decrypting the traffic for us.
+            self._mb = SungrowModbusTcpClient(self._ip, self._port,
+                                              framer=ModbusSocketFramer, timeout=1,
+                                              RetryOnEmpty=True, retries=1)
+        else:
+            self._mb = ModbusTcpClient(self._ip, self._port,
+                                       framer=ModbusSocketFramer, timeout=1,
+                                       RetryOnEmpty=True, retries=1)
 
     def add_monitor_register(self, table, addr):
         # Accepts a modbus register and table to monitor
@@ -43,10 +54,13 @@ class modbus_interface():
             for k in sorted(self._tables[table]):
                 group = int(k) - int(k) % DEFAULT_SCAN_BATCHING
                 if (start < group):
-                    values = self._scan_value_range(table, group, DEFAULT_SCAN_BATCHING)
-                    for x in range(0, DEFAULT_SCAN_BATCHING):
-                        key = group + x
-                        self._values[table][key] = values[x]
+                    try:
+                        values = self._scan_value_range(table, group, DEFAULT_SCAN_BATCHING)
+                        for x in range(0, DEFAULT_SCAN_BATCHING):
+                            key = group + x
+                            self._values[table][key] = values[x]
+                    except ValueError as e:
+                        logging.exception("{}".format(e))
                     start = group + DEFAULT_SCAN_BATCHING-1
         self._process_writes()
 
@@ -97,12 +111,18 @@ class modbus_interface():
                     self._mb.write_register(addr, new_value, unit=0x01)
                 sleep(DEFAULT_WRITE_SLEEP_S)
         except Exception as e:
+            # BUG catch only the specific exception that means pymodbus failed to write to a register
+            # the modbus device doesn't support, not an error at the TCP layer.
             logging.exception("Failed to write to modbus device: {}".format(e))
         finally:
             self._writing = False
 
     def _scan_value_range(self, table, start, count):
+        result = None
         if table == 'input':
-            return self._mb.read_input_registers(start, count, unit=0x01).registers
+            result = self._mb.read_input_registers(start, count, unit=0x01)
         elif table == 'holding':
-            return self._mb.read_holding_registers(start, count, unit=0x01).registers
\ No newline at end of file
+            result = self._mb.read_holding_registers(start, count, unit=0x01)
+        if isinstance(result, exceptions.ModbusIOException):
+            raise ValueError("Failed to read {} table registers from {} to {}".format(table, start, start+count))
+        return result.registers
\ No newline at end of file
